"""
Authentication Configuration

Authentication-specific configuration settings for CodeCheq.
"""

import os
import re
from typing import Optional
from urllib.parse import urlparse


class AuthConfig:
    """Authentication configuration settings."""

    # Whitelisted domains for token portal
    ALLOWED_DOMAINS = {
        "aioniq.ai",
        "localhost",  # Allow localhost for development
        "127.0.0.1",  # Allow localhost IP for development
    }

    def __init__(
        self,
        api_token: Optional[str] = None,
        token_portal_url: Optional[str] = None,
        cache_duration: int = 300,
        timeout: int = 30,
    ):
        """Initialize authentication configuration.

        Args:
            api_token: API token for authentication
            token_portal_url: URL of the token portal
            cache_duration: Token verification cache duration in seconds
            timeout: Request timeout in seconds
        """
        self.api_token = api_token or os.getenv("CODECHEQ_API_TOKEN")
        
        # Get token portal URL and validate domain
        raw_url = token_portal_url or os.getenv("CODECHEQ_TOKEN_PORTAL_URL", "http://localhost:5000")
        self.token_portal_url = self._validate_and_normalize_url(raw_url)
        
        self.cache_duration = cache_duration
        self.timeout = timeout
        self.require_auth = True

    def _validate_and_normalize_url(self, url: str) -> str:
        """Validate that the URL domain is in the whitelist and normalize the URL.
        
        Args:
            url: The URL to validate
            
        Returns:
            Normalized URL if valid
            
        Raises:
            ValueError: If the domain is not in the whitelist
        """
        try:
            # Add scheme if missing
            if not url.startswith(('http://', 'https://')):
                # Default to https for aioniq.ai, http for localhost
                if 'aioniq.ai' in url:
                    url = f"https://{url}"
                else:
                    url = f"http://{url}"
            
            parsed = urlparse(url)
            domain = parsed.netloc.lower()
            
            # Remove port if present for domain checking
            if ':' in domain:
                domain = domain.split(':')[0]
            
            # Check if domain is in whitelist
            if domain not in self.ALLOWED_DOMAINS:
                raise ValueError(
                    f"Domain '{domain}' is not in the allowed whitelist. "
                    f"Only the following domains are allowed: {', '.join(sorted(self.ALLOWED_DOMAINS))}. "
                    f"Please use a valid aioniq.ai URL or localhost for development."
                )
            
            return url.rstrip('/')
            
        except Exception as e:
            if isinstance(e, ValueError):
                raise
            raise ValueError(f"Invalid URL format: {url}. Error: {str(e)}")

    def has_token(self) -> bool:
        """Check if an API token is configured."""
        return bool(self.api_token)

    def is_valid(self) -> bool:
        """Check if the configuration is valid."""
        if not self.has_token():
            return False
        return True

    def get_validation_errors(self) -> list:
        """Get list of validation errors."""
        errors = []
        
        if not self.has_token():
            errors.append("Authentication is required but no API token is provided")
        
        if self.has_token() and not self.api_token.startswith("sk-"):
            errors.append("API token must start with 'sk-' prefix")
        
        return errors

    def to_dict(self) -> dict:
        """Convert configuration to dictionary."""
        return {
            "api_token": self.api_token,
            "token_portal_url": self.token_portal_url,
            "cache_duration": self.cache_duration,
            "timeout": self.timeout,
            "has_token": self.has_token(),
            "is_valid": self.is_valid(),
            "allowed_domains": list(self.ALLOWED_DOMAINS)
        } 